package dao;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;

import model.Tag;
import model.TagFile;
import exception.PersistenceException;
import factory.JDBCDAOFactory;

public class JDBCTagFileDAO implements TagFileDAO {

	private Statement stm;
	private PreparedStatement stmt;
	private ResultSet rs; 
	private Connection conn;
	
	public enum Sql {
		RECUPERAR_TODOS("SELECT tf.id_File, tf.id_tag, t.name_tag FROM smsd.tagfile tf, smsd.tag t " +
						"where tf.id_tag = t.id_tag"),
		
		SALVAR("INSERT INTO TAGFILE (ID_TAG, ID_FILE) VALUES(?, ?)"),
		
		REMOVER("DELETE FROM TAGFILE WHERE ID_TAG = ? AND ID_FILE = ?");
		
		Sql(String sql) {
			this.sql = sql;
		}
		
		public String toString() {
			return sql;
		}
		
		private String sql;
	}
	
	public JDBCTagFileDAO() {
		this.openPersistence();
	}
	
	/**
	 * Abre a persistencia
	 */
	@SuppressWarnings("unchecked")
	public <T> T openPersistence() {
		conn = ((Connection) JDBCDAOFactory.getInstance().openConnection());
		if (stm == null)
			try {
				stm = conn.createStatement();
			} catch (SQLException e) {
				e.printStackTrace();
			}
		return (T) stm;
	}

	/**
	 * Fecha a persistencia
	 */
	public <T> void closePersistence(T persistence) {
		if (persistence instanceof ResultSet) {
			rs = (ResultSet) persistence;
			try {
				if (rs != null)
					rs.close();
				
				if (stmt != null) 
					stmt.close();
				
				if (stm != null) 
					stm.close();
				
			} catch (SQLException e) {
				e.printStackTrace();
			}
		}
	}
	
	/**
	 * Remove o TagFile
	 * @param	codigoTag			id da Tag
	 * @param	codigoDataFile		id do DataFile
	 */
	public void remover(Integer idTag, Integer idDataFile)
			throws PersistenceException {
		Tag t = null;		
		try {
			//t = this.recuperarPorId(codigoTag);			
			stmt = conn.prepareStatement(Sql.REMOVER.toString());
			stmt.setInt(1, idTag);
			stmt.setInt(2, idDataFile);
			if (t != null) {
				stmt.executeUpdate();
				//conn.commit();
			}
		} catch (SQLException e) {
			e.printStackTrace();
		}
	}

	public void salvar(TagFile tf) throws PersistenceException {
		try {
			stmt = conn.prepareStatement(Sql.SALVAR.toString());
			stmt.setString(1, tf.getIdTag().toString());
			stmt.setString(2, tf.getIdFile().toString());
			stmt.executeUpdate();
	
		} catch (SQLException e) {
			System.out.println(e.getMessage());
		
		}
	}

	public ArrayList<TagFile> recuperarTodos() throws PersistenceException {
		try {
			rs = stm.executeQuery(Sql.RECUPERAR_TODOS.toString());
		} catch (SQLException e) {
			e.printStackTrace();
		}
		ArrayList<TagFile> tagsfiles = new ArrayList<TagFile>();
		try {
			while(rs.next()) {
				Integer idTag = rs.getInt("ID_TAG");
				Integer idFile = rs.getInt("ID_FILE");
				String nameTag = rs.getString("NAME_TAG");
				TagFile tf = new TagFile();

				tf.setIdTag(idTag);
				tf.setIdFile(idFile);
				tf.setNameTag(nameTag);
			
				tagsfiles.add(tf);
			}
		} catch (SQLException e) {
			e.printStackTrace();
		}
		return tagsfiles;
	}

}
