package factory;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;

import dao.DataFileDAO;
import dao.JDBCDataFileDAO;
import dao.JDBCTagDAO;
import dao.JDBCTagFileDAO;
import dao.TagDAO;
import dao.TagFileDAO;

public class JDBCDAOFactory extends DAOFactory {
	
	private Connection conn;
	private static JDBCDAOFactory instance = new JDBCDAOFactory();	
	
	public static final String DRIVER_NAME = "com.mysql.jdbc.Driver";
	public static final String DATABASE_NAME = "smsd";
	public static final String SERVER_NAME = "";
	public static final String STRING_CONNECTION = "jdbc:mysql://" + SERVER_NAME + "/" + DATABASE_NAME;
	public static final String USER_NAME = "root";
	public static final String PASSWORD = "123o45b";	
	
	private JDBCDAOFactory() {
		conn = this.openConnection();
	}
	
	public static JDBCDAOFactory getInstance() {
		return instance;
	}
	
	@SuppressWarnings("unchecked")
	public <T> T getFactory() {
		return (T) conn;
	}
	
	@SuppressWarnings("unchecked")
	@Override
	public <T> T openConnection() {
		try {
			Class.forName(DRIVER_NAME);
			conn = DriverManager.getConnection(STRING_CONNECTION, USER_NAME, PASSWORD);
			return (T) conn;
		} catch(ClassNotFoundException e) {
			e.printStackTrace();
		} catch(SQLException e) {
			e.printStackTrace();
		}
		return null;
	}

	@Override
	public <T> void closeConnection(T connection) {
		if (connection instanceof Connection) {
			conn = (Connection) connection;
			if (conn != null)
				try {
					conn.close();
				} catch (SQLException e) {
					e.printStackTrace();
				}
		}			
	}
	
	@Override
	public TagDAO getTagDAO() {
		return new JDBCTagDAO();
	}
 
	@Override
	public DataFileDAO getDataFileDAO() {
		return new JDBCDataFileDAO();
	}

	@Override
	public TagFileDAO getTagFileDAO() {
		return new JDBCTagFileDAO();
	}
	
}
