package model;

import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;
import java.util.Vector;

import exception.PersistenceException;

/**
 * A classe Core  o elemento principal do SMSD. 
 * @author  Daniel Bento
 * @version 1.0, May 2000
 */
public class Core {


	private MemoryManager mm = new MemoryManager();
	private PersistenceManager pm = new PersistenceManager();
	
	
	public Core() {
		this.mm.loadDataFiles(pm.datafileDAO);
		this.mm.loadTags(pm.tagDAO);
		this.mm.loadTagsFiles(pm.tagfileDAO);
		this.mm.asociateTagsFiles();
		
	}
	
	
	
	/**
	 * Adiciona a Tag t ao DataFile df
	 * @param   t		Tag a ser adicionada
	 * @param   df		DataFile
	 */
	public synchronized void addTagToDataFile(Tag t, DataFile df) {
		if (t != null && df != null) {
			if (!t.contemDataFile(df))
				try {
					TagFile tf =
						new TagFile(t.getId(), t.getName(), df.getId());
					
					pm.tagfileDAO.salvar(tf);
					this.mm.tagsfiles.add(tf);
					this.mm.linkTagToDF(t, df);

				}
				catch(Exception e) {
					e.printStackTrace();
				}
		}
	}
	
	
	/**
	 * Adiciona cada Tag em tags a cada DataFile em dfs 
	 * @param   tags	Vector de Tags
	 * @param	dfs		Vector de DataFiles
	 */
	private synchronized void addTagsToDataFiles(Vector<Tag> tags, Vector<DataFile> dfs) {
		for (Tag t : tags) {
			for (DataFile df : dfs) {
				if (!t.contemDataFile(df))
					try {
						TagFile tf =
							new TagFile(t.getId(), t.getName(), df.getId());
						
						pm.tagfileDAO.salvar(tf);
						this.mm.tagsfiles.add(tf);
						this.mm.linkTagToDF(t, df);

					}
					catch(Exception e) {
						System.out.println(e);
					}
				
			}
		}
		
	}
	
	
	/**
	 * Obtm a tag de nome nomeTag
	 * @param   nomeTag		Nome da Tag
	 * @return	A Tag de nome nomeTag
	 * 			null, caso a Tag no exista
	 */
	private synchronized Tag getTagByName(String nomeTag) {
		Tag t = null;
		t = this.mm.tags.get(nomeTag);
		return t;
	}
	
	/**
	 * Adiciona uma Tag ao sistema, caso a mesma no exista
	 * @param   nameTag		Nome da Tag
	 * @return	A Tag de nome nameTag
	 */
	public synchronized Tag createTag(String nameTag) {
		Tag tag = this.getTagByName(nameTag);
		if (tag == null)
			try {
				tag = new Tag();
				Integer chaveGerada = null;
				tag.setName(nameTag);
				chaveGerada = pm.tagDAO.salvar(tag);
				tag.setId(chaveGerada);
				this.mm.tags.put(tag.getName(), tag);
				System.out.println(chaveGerada);
				
			} catch (PersistenceException e) {
				System.out.println(e.getMessage());
			}
		else {
			System.out.println("Tag j existente: " + tag.getName());
		}
		return tag;
	}
	
	/**
	 * Remove uma Tag do sistema, caso a mesma exista
	 * @param   nameTag		Nome da Tag a ser removida
	 */
	public synchronized void removeTag(String nameTag) {
		Tag tag = this.getTagByName(nameTag);
		if (tag != null)
			try {
				pm.tagDAO.remover(tag.getId());
				this.mm.tags.remove(tag.getName());

				
			} catch (PersistenceException e) {
				System.out.println(e.getMessage());
			}
		else {
			System.out.println("Tag no encontrada: " + nameTag);
		}
	}
	
	/**
	 * Remove um DataFile do sistema, caso o mesma exista
	 * @param   path		Caminho do DataFile
	 */
	public synchronized void removeDataFile(String path) {
		DataFile df = this.getDataFileByPath(path);
		if (df != null)
			try {
				pm.datafileDAO.remover(df.getId());
				this.mm.datafiles.remove(df.getId());
		

				
			} catch (PersistenceException e) {
				System.out.println(e.getMessage());
			}
		else {
			System.out.println("DataFile no encontrado: " + path);
		}
	}
	
	/**
	 * Remove um TagFile do sistema, caso o mesma exista
	 * @param   nameTag		Nome da Tag
	 * @param	path		Caminho do Arquivo
	 */
	public synchronized void removeTagFile(String nameTag, String path) {
		DataFile df = this.getDataFileByPath(path);
		Tag t = this.getTagByName(nameTag);
		TagFile tf = this.mm.getTagFile(t.getId(), df.getId());
		if (tf != null)
			try {
				pm.tagfileDAO.remover(tf.getIdTag(), tf.getIdFile());
				this.mm.tagsfiles.remove(tf);
				this.mm.datafiles.remove(df.getId());
				
			} catch (PersistenceException e) {
				System.out.println(e.getMessage());
			}
		else {
			System.out.println("TagFile no encontrado: " + path);
		}
	}
	
	/**
	 * Adiciona um DataFile ao sistema, caso o mesmo no exista
	 * @param   path	Caminho do DataFile
	 * @param   type	Tipo do DataFile
	 * @return	O DataFile de caminho path
	 */
	public synchronized DataFile createDataFile(String path, String type) {
		DataFile datafile = this.getDataFileByPath(path);
		
		if (datafile == null)
			try {
				datafile = new DataFile();
				Integer chaveGerada = null;
				datafile.setPath(path);
				datafile.setType(type);
				chaveGerada = pm.datafileDAO.salvar(datafile);
				datafile.setId(chaveGerada);
				this.mm.datafiles.put(datafile.getId().toString(), datafile);
				
				System.out.println(chaveGerada);
				
			} catch (PersistenceException e) {
				System.out.println(e.getMessage());
			}
		else {
			System.out.println("DataFile j existente: " + datafile.getPath());
		}
		return datafile;
	}
	
	/**
	 * Obtm o DataFile de id idDataFile
	 * @param   idDataFile	id do DataFile
	 * @return	O DataFile de id idDataFile
	 * 			null, caso o DataFile no exista
	 */
	public synchronized DataFile getDataFileById(Integer idDataFile) {
		DataFile df = null;
		df = this.mm.datafiles.get(idDataFile);
		return df;
	}
	
	
	/**
	 * Obtm o DataFile de caminho pathDataFile
	 * @param   pathDataFile	caminho do DataFile
	 * @return	O DataFile de caminho pathDatafile
	 * 			null, caso o DataFile no exista
	 */
	public synchronized DataFile getDataFileByPath(String pathDataFile) {
		DataFile df = null;
		Set<String> set = this.mm.datafiles.keySet();
		Iterator<String> it = set.iterator();
		while (it.hasNext()) {
			String key = it.next();
			if (this.mm.datafiles.get(key).getPath().compareTo(pathDataFile) == 0) {
				df = this.mm.datafiles.get(key);
				break;
			}
		}
		return df;
	}
	


	/**
	 * Calcula a unio dos DataFiles pertencentes a um conjunto de Tags
	 * @param   tags	Vector (conjunto) de Tags
	 * @return	Um conjunto de DataFiles (Vector) com os elementos
	 * 			da unio de tags
	 * 			null, caso no exista DataFiles na unio
	 */
	private synchronized Vector<DataFile> unionTags(Vector<Tag> tags) {
		HashSet<DataFile> dfs = new HashSet<DataFile>();
		Vector<DataFile> vdfs = new Vector<DataFile>();
		for (Tag t : tags) {
			for (DataFile df : t.getDataFiles()) {
				dfs.add(df);
			}
		}
		vdfs.addAll(dfs);
		return vdfs;
	}
	
	/**
	 * Calcula a unio das Tags pertencentes a um conjunto de DataFiles
	 * @param   dfs		Vector (conjunto) de DataFiles
	 * @return	Um conjunto de Tags (Vector) com os elementos
	 * 			da unio de DataFiles
	 * 			null, caso no exista Tags na unio
	 */
	private synchronized Vector<Tag> unionDFs(Vector<DataFile> dfs) {
		HashSet<Tag> tags = new HashSet<Tag>();
		Vector<Tag> vtags = new Vector<Tag>();
				
		for (DataFile df : dfs) {
			for (Tag t : df.getTags()) {
				tags.add(t);
			}
		}
		vtags.addAll(tags);
		return vtags;
	}
	
	
	/**
	 * Calcula a interseco dos DataFiles pertencentes a um conjunto de Tags
	 * @param   tags	Vector (conjunto) de Tags
	 * @return	Um conjunto de DataFiles (Vector) com os elementos
	 * 			da interseco de tags
	 * 			null, caso no exista DataFiles na unio
	 */
	private synchronized Vector<DataFile> intersectionTags(Vector<Tag> tags) {
		Vector<DataFile> datafiles = new Vector<DataFile>();
		boolean contem = true;
		
		if (tags.size() > 0) {
			for (DataFile df : tags.get(0).getDataFiles()) {
				contem = true;
				for (Tag t : tags) {
					if (!t.contemDataFile(df)) {
						contem  = false;
						break;
					}
				}
				if (contem) datafiles.add(df);
			}
			
		}
		return datafiles;
	}
	
	/**
	 * Calcula a interseco das Tags pertencentes a um conjunto de DataFiles
	 * @param   dfs		Vector (conjunto) de DataFiles
	 * @return	Um conjunto de Tags (Vector) com os elementos
	 * 			da interseco de DataFiles
	 * 			null, caso no exista Tags na interseco
	 */
	private synchronized Vector<Tag> intersectionDFs(Vector<DataFile> dfs) {
		Vector<Tag> tags = new Vector<Tag>();
		boolean contem = true;
		
		if (dfs.size() > 0) {
			for (Tag t : dfs.get(0).getTags()) {
				contem = true;
				for (DataFile df : dfs) {
					if (!df.contemTag(t)) {
						contem  = false;
						break;
					}
				}
				if (contem) tags.add(t);
			}
		}
		return tags;
	}
	
	/**
	 * Obtm os DataFiles existentes na interseco das tags
	 * @param   tags	Comjunto de tags vlidas
	 * @return	Um conjunto de DataFiles
	 */
	private Vector<DataFile> buscaInterseccao(Vector<Tag> tags) {
		return this.intersectionTags(tags);
	}
	

	/**
	 * Realiza a busca por semelhana baseado nas tags de entrada
	 * @param   tags	Comjunto de tags vlidas
	 * @return	Um conjunto de DataFiles
	 */
	private Vector<DataFile> buscaSemelhanca(Vector<Tag> tags) {
		Integer conta = 0;
		Vector<DataFile> dfs = this.intersectionTags(tags);
		Vector<Tag> tagsSemelhantes = new Vector<Tag>();
		
		for (String strTag : this.mm.tags.keySet()) {
			conta = 0;
			for (DataFile df : dfs) {
				if (df.contemTag(strTag)) {
					conta++;
				}
			}
			if (conta != 0) {
				tagsSemelhantes.add(this.mm.tags.get(strTag));
			}
		}
		
		return this.unionTags(tagsSemelhantes);
	}
	
	

	/**
	 * strings  um vetor com nome de Tags
	 * Obtm um vetor de Tags com os nomes presentes em strings
	 * caso o nome no exista, a tag  desconsiderada
	 * @param   strings		Um conjunto com nome de Tags
	 * @return	Um conjunto de Tags (Vector) com os elementos
	 */
	private Vector<Tag> createTagsfromStrings (Vector<String> strings) {
		Vector<Tag> tags = new Vector<Tag>();
		for (String str : strings) {
			Tag t = this.getTagByName(str);
			if (t != null) tags.add(t);
		}
		return tags;
	}
	

	
	/**bug->e o tipo do datafile?
	 * strings  um vetor com nome de Tags
	 * Obtm um vetor de Tags com os nomes presentes em strings
	 * caso o nome no exista, a tag  desconsiderada
	 * @param   strings		Um conjunto com nome de Tags
	 * @return	Um conjunto de Tags (Vector) com os elementos
	 */
	private Vector<DataFile> createDataFilesfromStrings (Vector<String> strings) {
		Vector<DataFile> dfs = new Vector<DataFile>();
		for (String str : strings) {
			DataFile df = this.getDataFileByPath(str);
			if (df != null) dfs.add(df);
		}
		return dfs;
	}
	
	
	/**
	 * Wrapper para garantir a validade das tags na busca
	 * @param   tags	Comjunto de tags vlidas
	 * @return	Um conjunto de DataFiles
	 */
	public synchronized Vector<DataFile> buscaInterseccaoWrapper(Vector<String> args) {
		if (args != null) {
			Vector<Tag> tags = this.createTagsfromStrings(args);
			return this.buscaInterseccao(tags);
		}
		else
			return null;
		
	}
	
	/**
	 * Wrapper para garantir a validade das tags na busca
	 * @param   tags	Comjunto de tags vlidas
	 * @return	Um conjunto de DataFiles
	 */
	public synchronized Vector<DataFile> buscaSemelhancaWrapper(Vector<String> args) {

		if (args != null) {
			Vector<Tag> tags = this.createTagsfromStrings(args);
			return this.buscaSemelhanca(tags);
		}
		else
			return null;
		
	}
	

}
