package model;

import java.util.Hashtable;
import java.util.Vector;


public class DataFile {

	private Integer id;
	private String path;
	private String type;
	
	Hashtable <String, Tag> tags = new Hashtable <String, Tag>();
	
	public DataFile(){}
	
	public DataFile(String path, String type) {
		this.path = path;
		this.type = type;
	}
	
	/**
	 * Verifica se o DataFile contm a Tag t
	 * @param   t	Tag
	 * @return	true, caso contenha a Tag t
	 * 			false, caso contrrio
	 */
	public boolean contemTag(Tag t) {
		return this.tags.containsKey(t.getName());
	}
	
	/**
	 * Verifica se o DataFile contm uma Tag de nome tagName
	 * @param	tagName		Nome da Tag
	 * @return	true, caso contenha a Tag
	 * 			false, caso contrrio
	 */
	public boolean contemTag(String tagName) {
		return this.tags.containsKey(tagName);
	}
	
	/**
	 * Adiciona a Tag ao DataFile
	 * @param	tag		Tag a ser adicionada
	 */
	public void addTag (Tag tag){
		this.tags.put(tag.getName(), tag);
	}

	/**
	 * Obtm o conjunto de Tags pertencentes ao DataFile
	 * @return	Vector (conjunto) de Tags do DataFile
	 */
	public Vector<Tag> getTags() {
		Vector<Tag> tags = new Vector<Tag>();
		for (String strTag : this.tags.keySet()) {
			tags.add(this.tags.get(strTag));
		}
		return tags;
	}

	/**
	 * Obtm o id do DataFile
	 * @return	O id do DataFile
	 */
	public Integer getId() {
		return id;
	}
	
	/**
	 * Define o id do DataFile
	 * @param	id		id do DataFile
	 */
	public void setId(Integer id) {
		this.id = id;
	}

	/**
	 * Obtm o caminho do DataFile
	 * @return	O caminho do DataFile
	 */
	public String getPath() {
		return path;
	}

	/**
	 * Define o caminho do DataFile
	 * @param	path	caminho do DataFile
	 */
	public void setPath(String path) {
		this.path = path;
	}

	/**
	 * Obtm o tipo do DataFile
	 * @return	O tipo do DataFile
	 */
	public String getType() {
		return type;
	}

	/**
	 * Define o tipo do DataFile
	 * @param	type	tipo do DataFile
	 */
	public void setType(String type) {
		this.type = type;
	}

	/**
	 * @return	A representao em String do DataFile
	 */
	public String toString() {
		return path + ";" + type;
	}
}
